@@ -73,12 +73,13 @@ namespace cxxtools
       };
 
       static FlusherThread* flusherThread;
+      static bool flusherThreadStarted;
 
     public:
       LoggerImpl(const std::string& c, log_level_type l)
         : Logger(c, l)
         { }
-      std::ostream& getAppender() const;
+      static std::ostream& getAppender();
       void logEnd(std::ostream& appender);
       static void doRotate();
       static void setFile(const std::string& fname);
@@ -88,10 +89,16 @@ namespace cxxtools
       static void setLoghost(const std::string& host, unsigned short int port);
   };
 
-  std::ostream& LoggerImpl::getAppender() const
+  std::ostream& LoggerImpl::getAppender()
   {
-    if (outfile.is_open())
+    if (!fname.empty())
     {
+      if (!outfile.is_open())
+      {
+        outfile.clear();
+        outfile.open(fname.c_str(), std::ios::out | std::ios::app);
+      }
+
       if (maxfilesize > 0)
       {
         if (counter.getCount() > maxfilesize)
@@ -114,6 +121,11 @@ namespace cxxtools
   {
     if (flusherThread == 0)
       appender.flush();
+    else if (!flusherThreadStarted && !fname.empty())
+    {
+      flusherThread->create();
+      flusherThreadStarted = true;
+    }
   }
 
   std::string LoggerImpl::mkfilename(unsigned idx)
@@ -155,37 +167,37 @@ namespace cxxtools
   unsigned LoggerImpl::maxfilesize = 0;
   unsigned LoggerImpl::maxbackupindex = 0;
   LoggerImpl::FlusherThread* LoggerImpl::flusherThread = 0;
+  bool LoggerImpl::flusherThreadStarted = false;
 
   struct timespec LoggerImpl::FlusherThread::flushDelay;
 
   void LoggerImpl::setFile(const std::string& fname_)
   {
     fname = fname_;
-    outfile.clear();
-    outfile.open(fname_.c_str(), std::ios::out | std::ios::app);
-    counter.resetCount(outfile.tellp());
+
+    struct stat s;
+    int ret = stat(fname_.c_str(), &s);
+    counter.resetCount(ret == 0 ? s.st_size : 0);
   }
 
   void LoggerImpl::setFlushDelay(unsigned ms)
   {
     FlusherThread::setFlushDelay(ms);
-    if (ms > 0 && flusherThread == 0 && outfile.is_open())
-    {
+    if (ms > 0 && flusherThread == 0 && !fname.empty())
       flusherThread = new FlusherThread(outfile);
-      flusherThread->create();
-    }
     else if (ms == 0)
       flusherThread = 0;
   }
 
   void LoggerImpl::setLoghost(const std::string& host, unsigned short int port)
   {
-    loghost.connect(host.c_str(), port);
+    loghost.connect(host.c_str(), port, true);
   }
 
   RWLock Logger::rwmutex;
   Mutex Logger::mutex;
   Logger::log_level_type Logger::std_level = LOG_LEVEL_ERROR;
+  bool Logger::enabled = false;
 
   namespace
   {
@@ -255,6 +267,9 @@ namespace cxxtools
 
   Logger* Logger::getLogger(const std::string& category)
   {
+    if (!enabled)
+      return 0;
+
     // search existing Logger
     RdLock rdLock(rwmutex);
 
@@ -360,7 +375,7 @@ namespace cxxtools
       date[20] = '\0';
     }
 
-    std::ostream& out = getAppender();
+    std::ostream& out = LoggerImpl::getAppender();
 
     out.clear();
 
@@ -368,7 +383,7 @@ namespace cxxtools
         << loDigit(t.tv_usec / 100000) << loDigit(t.tv_usec / 10000)
         << loDigit(t.tv_usec / 1000) << loDigit(t.tv_usec / 100)
         << loDigit(t.tv_usec / 10)
-        << " [" << pthread_self() << "] "
+        << " [" << getpid() << '.' << pthread_self() << "] "
         << level << ' '
         << category << " - ";
 
@@ -386,27 +401,30 @@ namespace cxxtools
         : logger(logger_),
           level(level_)
           { }
+
       std::ostream& out()     { return msg; }
-      std::string str()       { return msg.str(); }
-      Logger* getLogger()     { return logger; }
-      const char* getLevel()  { return level; }
+      void flush()
+      {
+        MutexLock lock(Logger::mutex);
+
+        std::ostream& out(logger->logentry(level));
+        out << msg.str() << '\n';
+
+        logger->logEnd(out);
+      }
   };
 
   LogMessage::LogMessage(Logger* logger, const char* level)
     : impl(new LogMessageImpl(logger, level))
     { }
 
+  void LogMessage::flush()
+  {
+    impl->flush();
+  }
+
   LogMessage::~LogMessage()
   {
-    MutexLock lock(Logger::mutex);
-
-    Logger* logger = impl->getLogger();
-
-    std::ostream& out(logger->logentry(impl->getLevel()));
-    out << impl->str() << '\n';
-
-    logger->logEnd(out);
-
     delete impl;
   }
 
@@ -458,16 +476,26 @@ namespace cxxtools
 
 void log_init_cxxtools(cxxtools::Logger::log_level_type level)
 {
+  cxxtools::Logger::setEnabled(false);
+
   cxxtools::Logger::setRootLevel(level);
+
+  cxxtools::Logger::setEnabled(true);
   cxxtools::reinitializeLoggers();
 }
 
 void log_init_cxxtools(const std::string& propertyfilename)
 {
-  cxxtools::Logger::setRootLevel(cxxtools::Logger::LOG_LEVEL_ERROR);
+  cxxtools::Logger::setEnabled(false);
   cxxtools::getBaseLoggers().clear();
 
   std::ifstream in(propertyfilename.c_str());
+  log_init_cxxtools(in);
+}
+
+void log_init_cxxtools(std::istream& in)
+{
+  cxxtools::Logger::setEnabled(true);
 
   enum state_type {
     state_0,
@@ -487,6 +515,7 @@ void log_init_cxxtools(const std::string& propertyfilename)
     state_maxbackupindex,
     state_flushdelay0,
     state_flushdelay,
+    state_disable,
     state_skip
   };
   
@@ -540,6 +569,8 @@ void log_init_cxxtools(const std::string& propertyfilename)
           state = state_maxbackupindex0;
         else if (ch == '=' && token == "FLUSHDELAY")
           state = state_flushdelay0;
+        else if (ch == '=' && token == "DISABLED")
+          state = state_disable;
         else if (ch == '\n')
           state = state_0;
         else if (std::isspace(ch))
@@ -566,6 +597,8 @@ void log_init_cxxtools(const std::string& propertyfilename)
           state = state_maxbackupindex0;
         else if (ch == '=' && token == "FLUSHDELAY")
           state = state_flushdelay0;
+        else if (ch == '=' && token == "DISABLE")
+          state = state_disable;
         else if (ch == '\n')
           state = state_0;
         else if (!std::isspace(ch))
@@ -736,6 +769,16 @@ void log_init_cxxtools(const std::string& propertyfilename)
           state = (ch == '\n' ? state_0 : state_skip);
         break;
 
+      case state_disable:
+        if (ch == '1' || ch == 't' || ch == 'T' || ch == 'y' || ch == 'Y')
+        {
+          cxxtools::Logger::setEnabled(false);
+          state = state_skip;
+        }
+        else if (ch != ' ' && ch != '\t')
+          state = (ch == '\n' ? state_0 : state_skip);
+        break;
+
       case state_skip:
         if (ch == '\n')
           state = state_0;
